import * as i0 from '@angular/core';
import { Injectable, EventEmitter, Component, ViewChild, Input, Output, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import html2canvas from 'html2canvas';
import { of, from, Subject } from 'rxjs';
import { take, tap } from 'rxjs/operators';

class NgxCaptureService {
    constructor() { }
    getImage(screen, fullCapture, cropDimensions) {
        let options = {
            logging: false,
        };
        if (!fullCapture &&
            cropDimensions.width > 10 &&
            cropDimensions.height > 10) {
            options = Object.assign(Object.assign({}, options), cropDimensions);
        }
        else if (!fullCapture) {
            return of(null);
        }
        return from(html2canvas(screen, options)
            .then((canv) => {
            const img = canv.toDataURL("image/png");
            return img;
        }, (err) => {
            throw new Error(err);
        })
            .catch((res) => {
            throw new Error(res);
        }));
    }
    // TODO
    downloadImage(img) {
        // this.canvas.nativeElement.src = img;
        // this.downloadLink.nativeElement.href = img;
        // this.downloadLink.nativeElement.download = 'test.png';
        // this.downloadLink.nativeElement.click();
    }
}
NgxCaptureService.ɵprov = i0.ɵɵdefineInjectable({ factory: function NgxCaptureService_Factory() { return new NgxCaptureService(); }, token: NgxCaptureService, providedIn: "root" });
NgxCaptureService.decorators = [
    { type: Injectable, args: [{
                providedIn: "root",
            },] }
];
NgxCaptureService.ctorParameters = () => [];

class NgxCaptureComponent {
    constructor(captureService) {
        this.captureService = captureService;
        this.resultImage = new EventEmitter();
        this.isDrawing = false;
        this.cropDimensions = {
            x: 0,
            y: 0,
            width: 0,
            height: 0
        };
        this.mouse = {
            x: 0,
            y: 0,
            startX: 0,
            startY: 0
        };
        this.destroy$ = new Subject();
    }
    ngOnInit() {
        // if (!this.captureZone) {
        //   console.warn('"captureZone" is not set');
        //   return;
        // }
        setTimeout(() => {
            this.rect = this.rectangle.nativeElement;
            this.captureZone = this.overlay.nativeElement;
            this.captureZone.onmousedown = (e) => this.startCapture(e);
            this.captureZone.onmousemove = (e) => this.drawRect(e);
            this.captureZone.onmouseup = () => this.endCapture();
        }, 2000);
    }
    setRectangle() {
        this.rect.style.left = this.cropDimensions.x + 'px';
        this.rect.style.top = this.cropDimensions.y + 'px';
        this.rect.style.width = this.cropDimensions.width + 'px';
        this.rect.style.height = this.cropDimensions.height + 'px';
    }
    startCapture(e) {
        this.setMousePosition(e);
        this.isDrawing = true;
        this.mouse.startX = this.mouse.x;
        this.mouse.startY = this.mouse.y;
        this.cropDimensions = {
            x: this.mouse.x,
            y: this.mouse.y,
            width: 0,
            height: 0,
        };
        this.setRectangle();
        this.captureZone.style.cursor = 'crosshair';
    }
    drawRect(e) {
        if (this.isDrawing) {
            this.setMousePosition(e);
            this.cropDimensions = {
                x: (this.mouse.x - this.mouse.startX < 0) ? this.mouse.x : this.mouse.startX,
                y: (this.mouse.y - this.mouse.startY < 0) ? this.mouse.y : this.mouse.startY,
                width: Math.abs(this.mouse.x - this.mouse.startX),
                height: Math.abs(this.mouse.y - this.mouse.startY),
            };
            this.setRectangle();
        }
    }
    setMousePosition(e) {
        // eslint-disable-next-line import/no-deprecated
        const ev = e || window.event; // Moz || IE
        if (ev.pageX) { // Moz
            this.mouse.x = ev.pageX + window.pageXOffset;
            this.mouse.y = ev.pageY + window.pageYOffset;
        }
        else if (ev.clientX) { // IE
            this.mouse.x = ev.clientX + document.body.scrollLeft;
            this.mouse.y = ev.clientY + document.body.scrollTop;
        }
    }
    endCapture() {
        this.captureZone.style.cursor = 'default';
        this.isDrawing = false;
        this.captureService
            .getImage(this.target, false, this.cropDimensions)
            .pipe(take(1), tap(img => {
            this.resultImage.emit(img);
        })).subscribe();
        this.cropDimensions = {
            x: 0,
            y: 0,
            width: 0,
            height: 0
        };
        this.setRectangle();
    }
}
NgxCaptureComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-capture',
                template: `
  <ng-content></ng-content>
  <div class="overlay" #over>
    <div class="rectangle" #rect></div>
  </div>
  `,
                styles: [".overlay{top:0px;left:0px;position:absolute;background-color:#cccccc1a;width:100vw;height:100vh}.rectangle{border:1px solid #ff0000;position:absolute}\n"]
            },] }
];
NgxCaptureComponent.ctorParameters = () => [
    { type: NgxCaptureService }
];
NgxCaptureComponent.propDecorators = {
    rectangle: [{ type: ViewChild, args: ['rect', { static: true },] }],
    overlay: [{ type: ViewChild, args: ['over', { static: true },] }],
    target: [{ type: Input }],
    resultImage: [{ type: Output }]
};

class NgxCaptureModule {
}
NgxCaptureModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NgxCaptureComponent],
                imports: [],
                exports: [NgxCaptureComponent],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/*
 * Public API Surface of ngx-capture
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxCaptureComponent, NgxCaptureModule, NgxCaptureService };
//# sourceMappingURL=ngx-capture.js.map
